@@ -42,8 +42,8 @@
 
 #include <wtf/ASCIICType.h>
 
-#include <QDebug>
-#include <QtCore/QRegExp>
+#include "khtml_debug.h"
+#include <QRegExp>
 #include <QPaintDevice>
 
 // Hack for debugging purposes
@@ -116,18 +116,24 @@ static DOMString quoteStringIfNeeded(const DOMString &string)
     return isCSSTokenizerIdentifier(string) ? string : quoteString(string);
 }
 
+static inline bool isInitialOrInherit(const CSSValueImpl *value)
+{
+    const unsigned short t = value->cssValueType();
+    return (t == CSSValue::CSS_INHERIT || t == CSSValue::CSS_INITIAL);
+}
+
 CSSStyleDeclarationImpl::CSSStyleDeclarationImpl(CSSRuleImpl *parent)
     : StyleBaseImpl(parent)
 {
-    m_lstValues = 0;
-    m_node = 0;
+    m_lstValues = nullptr;
+    m_node = nullptr;
 }
 
 CSSStyleDeclarationImpl::CSSStyleDeclarationImpl(CSSRuleImpl *parent, QList<CSSProperty *> *lstValues)
     : StyleBaseImpl(parent)
 {
     m_lstValues = lstValues;
-    m_node = 0;
+    m_node = nullptr;
 }
 
 CSSStyleDeclarationImpl  &CSSStyleDeclarationImpl::operator= (const CSSStyleDeclarationImpl &o)
@@ -141,7 +147,7 @@ CSSStyleDeclarationImpl  &CSSStyleDeclarationImpl::operator= (const CSSStyleDecl
         qDeleteAll(*m_lstValues);
     }
     delete m_lstValues;
-    m_lstValues = 0;
+    m_lstValues = nullptr;
     if (o.m_lstValues) {
         m_lstValues = new QList<CSSProperty *>;
         QListIterator<CSSProperty *> lstValuesIt(*o.m_lstValues);
@@ -166,7 +172,7 @@ CSSValueImpl *CSSStyleDeclarationImpl::getPropertyCSSValue(const DOMString &prop
 {
     int propID = propertyID(propertyName);
     if (!propID) {
-        return 0;
+        return nullptr;
     }
     return getPropertyCSSValue(propID);
 }
@@ -230,11 +236,12 @@ DOMString CSSStyleDeclarationImpl::getPropertyValue(int propertyID) const
         return getLayeredShortHandValue(properties, 2);
     }
     case CSS_PROP_BACKGROUND: {
-        const int properties[6] = { CSS_PROP_BACKGROUND_IMAGE, CSS_PROP_BACKGROUND_REPEAT,
-                                    CSS_PROP_BACKGROUND_ATTACHMENT, CSS_PROP_BACKGROUND_POSITION_X,
-                                    CSS_PROP_BACKGROUND_POSITION_Y, CSS_PROP_BACKGROUND_COLOR
+        // 'clip' must come after 'origin' in this array
+        const int properties[9] = { CSS_PROP_BACKGROUND_IMAGE, CSS_PROP_BACKGROUND_REPEAT, CSS_PROP_BACKGROUND_ATTACHMENT,
+                                    CSS_PROP_BACKGROUND_POSITION_X, CSS_PROP_BACKGROUND_POSITION_Y, CSS_PROP_BACKGROUND_SIZE,
+                                    CSS_PROP_BACKGROUND_ORIGIN, CSS_PROP_BACKGROUND_CLIP, CSS_PROP_BACKGROUND_COLOR
                                   };
-        return getLayeredShortHandValue(properties, 6);
+        return getLayeredShortHandValue(properties, 9);
     }
     case CSS_PROP_BORDER: {
         const int properties[3][4] = {{
@@ -337,7 +344,7 @@ DOMString CSSStyleDeclarationImpl::getPropertyValue(int propertyID) const
         return getShortHandValue(properties, 3);
     }
     }
-    //qDebug() << "property not found:" << propertyID;
+    //qCDebug(KHTML_LOG) << "property not found:" << propertyID;
     return DOMString();
 }
 
@@ -365,19 +372,63 @@ DOMString CSSStyleDeclarationImpl::getCommonValue(const int *properties, int num
 
 DOMString CSSStyleDeclarationImpl::get4Values(const int *properties) const
 {
-    DOMString res;
+    // Assume the properties are in the order top, right, bottom, left.
+    QVector<DOMString> values(4);
     for (int i = 0; i < 4; ++i) {
-        if (!isPropertyImplicit(properties[i])) {
-            CSSValueImpl *value = getPropertyCSSValue(properties[i]);
-            if (!value) {   // apparently all 4 properties must be specified.
-                return DOMString();
-            }
-            if (i > 0) {
-                res += " ";
-            }
-            res += value->cssText();
+        CSSValueImpl *val = getPropertyCSSValue(properties[i]);
+        // All 4 properties must be specified.
+        if (!val || isInitialOrInherit(val)) {
+            return DOMString();
+        } else {
+            values[i] = val->cssText();
         }
     }
+
+    // Reduce shorthand.
+    if (values.at(1) == values.at(3)) { // right/left
+        values.remove(3);
+        if (values.at(0) == values.at(2)) { // top/bottom
+            values.remove(2);
+            if (values.at(0) == values.at(1)) {
+                values.remove(1);
+            }
+        }
+    }
+
+    DOMString res;
+    const int valuesSize = values.size();
+    for (int i = 0; i < valuesSize; ++i) {
+        if (!res.isNull()) {
+            res += " ";
+        }
+        res += values.at(i);
+    }
+
+    return res;
+}
+
+static inline DOMString posXYSize_string_helper(DOMString &bPosX, DOMString &bPosY, DOMString &bSize)
+{
+    DOMString res, position;
+
+    if (!bPosX.isEmpty() && !bPosY.isEmpty()) {
+        position = bPosX + DOMString(" ") + bPosY;
+    } else if (bPosX.isEmpty() && !bPosY.isEmpty()) {
+        position = DOMString("0% ") + bPosY;
+    } else if (!bPosX.isEmpty() && bPosY.isEmpty()) {
+        position = bPosX + DOMString(" 0%");
+    }
+
+    if (!bSize.isEmpty()) {
+        if (position.isEmpty()) {
+            res = DOMString("0% 0%") + DOMString(" / ") + bSize;
+        } else {
+            res = position + DOMString(" / ") + bSize;
+        }
+    } else {
+        res = position;
+    }
+
     return res;
 }
 
@@ -403,12 +454,13 @@ DOMString CSSStyleDeclarationImpl::getLayeredShortHandValue(const int *propertie
         }
     }
 
-    // Now stitch the properties together.  Implicit initial values are flagged as such and
-    // can safely be omitted.
+    // Now stitch the properties together.
+    // Implicit initial values are flagged as such and can safely be omitted.
     for (i = 0; i < numLayers; i++) {
         DOMString layerRes;
+        DOMString bPosX, bPosY, bSize;
         for (j = 0; j < number; j++) {
-            CSSValueImpl *value = 0;
+            CSSValueImpl *value = nullptr;
             if (values[j]) {
                 if (values[j]->isValueList()) {
                     value = static_cast<CSSValueListImpl *>(values[j])->item(i);
@@ -418,22 +470,41 @@ DOMString CSSStyleDeclarationImpl::getLayeredShortHandValue(const int *propertie
                     // Color only belongs in the last layer.
                     if (properties[j] == CSS_PROP_BACKGROUND_COLOR) {
                         if (i != numLayers - 1) {
-                            value = 0;
+                            value = nullptr;
                         }
                     } else if (i != 0) { // Other singletons only belong in the first layer.
-                        value = 0;
+                        value = nullptr;
                     }
                 }
             }
 
             if (value && !value->isImplicitInitialValue()) {
-                if (!layerRes.isNull()) {
-                    layerRes += " ";
+                // positionX,positionY,size should be handled separately in order
+                // to return a consistent and valid 'background' property string
+                if (properties[j] == CSS_PROP_BACKGROUND_POSITION_X) {
+                    bPosX = value->cssText();
+                } else if (properties[j] == CSS_PROP_BACKGROUND_POSITION_Y) {
+                    bPosY = value->cssText();
+                } else if (properties[j] == CSS_PROP_BACKGROUND_SIZE) {
+                    bSize = value->cssText();
+                } else {
+                    if (!layerRes.isNull()) {
+                        layerRes += " ";
+                    }
+                    layerRes += value->cssText();
                 }
-                layerRes += value->cssText();
             }
         }
 
+        // now add positionX,positionY,size
+        DOMString posXYSize = posXYSize_string_helper(bPosX, bPosY, bSize);
+        if (!posXYSize.isEmpty()) {
+            if (!layerRes.isNull()) {
+                layerRes += " ";
+            }
+            layerRes += posXYSize;
+        }
+
         if (!layerRes.isNull()) {
             if (!res.isNull()) {
                 res += ", ";
@@ -463,7 +534,7 @@ DOMString CSSStyleDeclarationImpl::getShortHandValue(const int *properties, int
 CSSValueImpl *CSSStyleDeclarationImpl::getPropertyCSSValue(int propertyID) const
 {
     if (!m_lstValues || m_lstValues->isEmpty()) {
-        return 0;
+        return nullptr;
     }
 
     QListIterator<CSSProperty *> lstValuesIt(*m_lstValues);
@@ -474,7 +545,7 @@ CSSValueImpl *CSSStyleDeclarationImpl::getPropertyCSSValue(int propertyID) const
             return current->value();
         }
     }
-    return 0;
+    return nullptr;
 }
 
 bool CSSStyleDeclarationImpl::isPropertyImplicit(int propertyID) const
@@ -499,7 +570,7 @@ bool CSSStyleDeclarationImpl::isPropertyImplicit(int propertyID) const
 
 struct PropertyLonghand {
     PropertyLonghand()
-        : m_properties(0)
+        : m_properties(nullptr)
         , m_length(0)
     {
     }
@@ -630,8 +701,9 @@ static void initShorthandMap(QHash<int, PropertyLonghand> &shorthandMap)
         CSS_PROP_BACKGROUND_POSITION_X,
         CSS_PROP_BACKGROUND_POSITION_Y,
         CSS_PROP_BACKGROUND_REPEAT,
-        CSS_PROP__KHTML_BACKGROUND_SIZE,
-        CSS_PROP_BACKGROUND_SIZE
+        CSS_PROP_BACKGROUND_SIZE,
+        CSS_PROP_BACKGROUND_ORIGIN,
+        CSS_PROP_BACKGROUND_CLIP
     };
     SET_SHORTHAND_MAP_ENTRY(shorthandMap, CSS_PROP_BACKGROUND, backgroundProperties);
 
@@ -657,13 +729,6 @@ static void initShorthandMap(QHash<int, PropertyLonghand> &shorthandMap)
         CSS_PROP_BORDER_BOTTOM_RIGHT_RADIUS
     };
     SET_SHORTHAND_MAP_ENTRY(shorthandMap, CSS_PROP_BORDER_RADIUS, borderRadiusProperties);
-    static const int prefixedBorderRadiusProperties[] = {
-        CSS_PROP__KHTML_BORDER_TOP_RIGHT_RADIUS,
-        CSS_PROP__KHTML_BORDER_TOP_LEFT_RADIUS,
-        CSS_PROP__KHTML_BORDER_BOTTOM_LEFT_RADIUS,
-        CSS_PROP__KHTML_BORDER_BOTTOM_RIGHT_RADIUS
-    };
-    SET_SHORTHAND_MAP_ENTRY(shorthandMap, CSS_PROP__KHTML_BORDER_RADIUS, prefixedBorderRadiusProperties);
 
     static const int markerProperties[] = {
         CSS_PROP_MARKER_START,
@@ -822,8 +887,8 @@ bool CSSStyleDeclarationImpl::setProperty(int id, const DOMString &value, bool i
     CSSParser parser(strictParsing);
     bool success = parser.parseValue(this, id, value, important);
     if (!success) {
-        // qDebug() << "CSSStyleDeclarationImpl::setProperty invalid property: [" << getPropertyName(id).string()
-        //<< "] value: [" << value.string() << "]"<< endl;
+        // qCDebug(KHTML_LOG) << "CSSStyleDeclarationImpl::setProperty invalid property: [" << getPropertyName(id).string()
+        //<< "] value: [" << value.string() << "]";
     } else {
         setChanged();
     }
@@ -879,7 +944,7 @@ DOMString CSSStyleDeclarationImpl::item(unsigned long index) const
 CSSRuleImpl *CSSStyleDeclarationImpl::parentRule() const
 {
     return (m_parent && m_parent->isRule()) ?
-           static_cast<CSSRuleImpl *>(m_parent) : 0;
+           static_cast<CSSRuleImpl *>(m_parent) : nullptr;
 }
 
 DOM::DOMString CSSStyleDeclarationImpl::cssText() const
@@ -890,8 +955,8 @@ DOM::DOMString CSSStyleDeclarationImpl::cssText() const
 
     DOMString result;
 
-    const CSSProperty *positionXProp = 0;
-    const CSSProperty *positionYProp = 0;
+    const CSSProperty *positionXProp = nullptr;
+    const CSSProperty *positionYProp = nullptr;
 
     QListIterator<CSSProperty *> lstValuesIt(*m_lstValues);
     while (lstValuesIt.hasNext()) {
@@ -947,7 +1012,7 @@ void CSSStyleDeclarationImpl::setCssText(const DOM::DOMString &text)
 
 bool CSSStyleDeclarationImpl::parseString(const DOMString &/*string*/, bool)
 {
-    // qDebug() << "WARNING: CSSStyleDeclarationImpl::parseString, unimplemented, was called";
+    // qCDebug(KHTML_LOG) << "WARNING: CSSStyleDeclarationImpl::parseString, unimplemented, was called";
     return false;
     // ###
 }
@@ -1251,7 +1316,7 @@ unsigned short CSSPrimitiveValueImpl::cssValueType() const
 bool CSSPrimitiveValueImpl::parseString(const DOMString &/*string*/, bool)
 {
     // ###
-    // qDebug() << "WARNING: CSSPrimitiveValueImpl::parseString, unimplemented, was called";
+    // qCDebug(KHTML_LOG) << "WARNING: CSSPrimitiveValueImpl::parseString, unimplemented, was called";
     return false;
 }
 
@@ -1398,10 +1463,10 @@ DOM::DOMString CSSPrimitiveValueImpl::cssText() const
 
 RectImpl::RectImpl()
 {
-    m_top = 0;
-    m_right = 0;
-    m_bottom = 0;
-    m_left = 0;
+    m_top = nullptr;
+    m_right = nullptr;
+    m_bottom = nullptr;
+    m_left = nullptr;
 }
 
 RectImpl::~RectImpl()
@@ -1508,7 +1573,7 @@ void PairImpl::setSecond(CSSPrimitiveValueImpl *second)
 CSSImageValueImpl::CSSImageValueImpl(const DOMString &url, StyleBaseImpl *style)
     : CSSPrimitiveValueImpl(url, CSSPrimitiveValue::CSS_URI)
 {
-    m_image = 0;
+    m_image = nullptr;
     const DOMString imgUrl = url.trimSpaces();
     if (!imgUrl.isEmpty()) {
         m_fullImageUrl = style->baseURL().resolved(QUrl(imgUrl.string())).toString();
@@ -1520,7 +1585,7 @@ CSSImageValueImpl::CSSImageValueImpl(const DOMString &url, StyleBaseImpl *style)
 CSSImageValueImpl::CSSImageValueImpl()
     : CSSPrimitiveValueImpl(CSS_VAL_NONE)
 {
-    m_image = 0;
+    m_image = nullptr;
     m_fullImageUrl.clear();
 }
 
@@ -1566,7 +1631,7 @@ FontFamilyValueImpl::FontFamilyValueImpl(const QString &string)
     const QString &available = KHTMLSettings::availableFamilies();
 
     parsedFontName = parsedFontName.toLower();
-    // qDebug() << "searching for face '" << parsedFontName << "'";
+    // qCDebug(KHTML_LOG) << "searching for face '" << parsedFontName << "'";
 
     int pos = available.indexOf(',' + parsedFontName + ',', 0, Qt::CaseInsensitive);
     if (pos == -1) {
@@ -1588,14 +1653,14 @@ FontFamilyValueImpl::FontFamilyValueImpl(const QString &string)
         int p = available.indexOf(',', pos);
         assert(p != -1);   // available is supposed to start and end with ,
         parsedFontName = available.mid(pos, p - pos);
-        // qDebug() << "going for '" << parsedFontName << "'";
+        // qCDebug(KHTML_LOG) << "going for '" << parsedFontName << "'";
     }
 
 #endif // !APPLE_CHANGES
 }
 
 FontValueImpl::FontValueImpl()
-    : style(0), variant(0), weight(0), size(0), lineHeight(0), family(0)
+    : style(nullptr), variant(nullptr), weight(nullptr), size(nullptr), lineHeight(nullptr), family(nullptr)
 {
 }
 
@@ -1676,7 +1741,7 @@ QString QuotesValueImpl::openQuote(int level) const
         return "";
     }
     level--; // increments are calculated before openQuote is called
-//     qDebug() << "Open quote level:" << level;
+//     qCDebug(KHTML_LOG) << "Open quote level:" << level;
     if (level < 0) {
         level = 0;
     } else if (level >= (int) levels) {
@@ -1690,7 +1755,7 @@ QString QuotesValueImpl::closeQuote(int level) const
     if (levels == 0) {
         return "";
     }
-//     qDebug() << "Close quote level:" << level;
+//     qCDebug(KHTML_LOG) << "Close quote level:" << level;
     if (level < 0) {
         level = 0;
     } else if (level >= (int) levels) {
