@@ -36,9 +36,9 @@
 #  endif
 #endif
 
-#include <QDebug>
+#include "khtml_debug.h"
 
-#include <QtCore/QHash>
+#include <QHash>
 #include <QPainter>
 #include <QFontDatabase>
 #include <QtGlobal>
@@ -613,7 +613,7 @@ CachedFontInstance::CachedFontInstance(CachedFontFamily *p, int sz):
 
     // Prepare metrics caches
     for (int c = 0; c < 256; ++c) {
-        rows[c] = 0;
+        rows[c] = nullptr;
     }
 
     ascent  = fm.ascent();
@@ -643,7 +643,7 @@ void CachedFontInstance::invalidate()
     // Cleanup metrics caches
     for (int c = 0; c < 256; ++c) {
         delete rows[c];
-        rows[c] = 0;
+        rows[c] = nullptr;
     }
 
     ascent  = fm.ascent();
@@ -692,7 +692,7 @@ void Font::update(int logicalDpiY) const
         const QList<int> pointSizes = family->sizes;
         // lets see if we find a nice looking font, which is not too far away
         // from the requested one.
-        // qDebug() << "khtml::setFontSize family = " << f.family() << " size requested=" << size;
+        // qCDebug(KHTML_LOG) << "khtml::setFontSize family = " << f.family() << " size requested=" << size;
         const float toPix = qMax(logicalDpiY, 96) / 72.0f;
 
         float diff = 1; // ### 100% deviation
@@ -703,7 +703,7 @@ void Font::update(int logicalDpiY) const
 
         for (; it != itEnd; ++it) {
             float newDiff = (((*it) * toPix) - float(size)) / float(size);
-            //qDebug() << "smooth font size: " << *it << " diff=" << newDiff;
+            //qCDebug(KHTML_LOG) << "smooth font size: " << *it << " diff=" << newDiff;
             if (newDiff < 0) {
                 newDiff = -newDiff;
             }
@@ -712,7 +712,7 @@ void Font::update(int logicalDpiY) const
                 bestSize = *it;
             }
         }
-        //qDebug() << "best smooth font size: " << bestSize << " diff=" << diff;
+        //qCDebug(KHTML_LOG) << "best smooth font size: " << bestSize << " diff=" << diff;
         if (bestSize != 0 && diff < 0.2) { // 20% deviation, otherwise we use a scaled font...
             size = static_cast<int>(bestSize * toPix);
         }
@@ -730,7 +730,7 @@ void Font::update(int logicalDpiY) const
 
     // small caps
     delete scFont;
-    scFont = 0;
+    scFont = nullptr;
 
     if (fontDef.smallCaps) {
         scFont = new QFont(cfi->f);
